<?php

namespace Litepie\Medium\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepie\Medium\Http\Requests\MediaTopicRequest;
use Litepie\Medium\Interfaces\MediaTopicRepositoryInterface;
use Litepie\Medium\Models\MediaTopic;

/**
 * Resource controller class for media_topic.
 */
class MediaTopicResourceController extends BaseController
{

    /**
     * Initialize media_topic resource controller.
     *
     * @param type MediaTopicRepositoryInterface $media_topic
     *
     * @return null
     */
    public function __construct(MediaTopicRepositoryInterface $media_topic)
    {
        parent::__construct();
        $this->repository = $media_topic;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepie\Medium\Repositories\Criteria\MediaTopicResourceCriteria::class);
    }

    /**
     * Display a list of media_topic.
     *
     * @return Response
     */
    public function index(MediaTopicRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepie\Medium\Repositories\Presenter\MediaTopicPresenter::class)
                ->$function();
        }

        $media_topics = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('medium::media_topic.names'))
            ->view('medium::media_topic.index', true)
            ->data(compact('media_topics', 'view'))
            ->output();
    }

    /**
     * Display media_topic.
     *
     * @param Request $request
     * @param Model   $media_topic
     *
     * @return Response
     */
    public function show(MediaTopicRequest $request, MediaTopic $media_topic)
    {

        if ($media_topic->exists) {
            $view = 'medium::media_topic.show';
        } else {
            $view = 'medium::media_topic.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('medium::media_topic.name'))
            ->data(compact('media_topic'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new media_topic.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MediaTopicRequest $request)
    {

        $media_topic = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('medium::media_topic.name')) 
            ->view('medium::media_topic.create', true) 
            ->data(compact('media_topic'))
            ->output();
    }

    /**
     * Create new media_topic.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MediaTopicRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $media_topic                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('medium::media_topic.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('medium/media_topic/' . $media_topic->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/medium/media_topic'))
                ->redirect();
        }

    }

    /**
     * Show media_topic for editing.
     *
     * @param Request $request
     * @param Model   $media_topic
     *
     * @return Response
     */
    public function edit(MediaTopicRequest $request, MediaTopic $media_topic)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('medium::media_topic.name'))
            ->view('medium::media_topic.edit', true)
            ->data(compact('media_topic'))
            ->output();
    }

    /**
     * Update the media_topic.
     *
     * @param Request $request
     * @param Model   $media_topic
     *
     * @return Response
     */
    public function update(MediaTopicRequest $request, MediaTopic $media_topic)
    {
        try {
            $attributes = $request->all();

            $media_topic->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('medium::media_topic.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('medium/media_topic/' . $media_topic->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('medium/media_topic/' . $media_topic->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the media_topic.
     *
     * @param Model   $media_topic
     *
     * @return Response
     */
    public function destroy(MediaTopicRequest $request, MediaTopic $media_topic)
    {
        try {

            $media_topic->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('medium::media_topic.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('medium/media_topic/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('medium/media_topic/' . $media_topic->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple media_topic.
     *
     * @param Model   $media_topic
     *
     * @return Response
     */
    public function delete(MediaTopicRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('medium::media_topic.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('medium/media_topic'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/medium/media_topic'))
                ->redirect();
        }

    }

    /**
     * Restore deleted media_topics.
     *
     * @param Model   $media_topic
     *
     * @return Response
     */
    public function restore(MediaTopicRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('medium::media_topic.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/medium/media_topic'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/medium/media_topic/'))
                ->redirect();
        }

    }

}
